@@ -58,6 +58,11 @@ sub cgitemplate ($$$;@) {
 	
 	my $template=template("page.tmpl");
 
+	my $topurl = $config{url};
+	if (defined $cgi && ! $config{w3mmode} && ! $config{reverse_proxy}) {
+		$topurl = $cgi->url;
+	}
+
 	my $page="";
 	if (exists $params{page}) {
 		$page=delete $params{page};
@@ -483,9 +488,6 @@ sub cgi (;$$) {
 sub cgierror ($) {
 	my $message=shift;
 
-	eval q{use HTML::Entities};
-	$message = encode_entities($message);
-
 	print "Content-type: text/html\n\n";
 	print cgitemplate(undef, gettext("Error"),
 		"<p class=\"error\">".gettext("Error").": $message</p>");
