@@ -356,8 +356,7 @@ sub editcomment ($$) {
 	my @page_types;
 	if (exists $IkiWiki::hooks{htmlize}) {
 		foreach my $key (grep { !/^_/ && isallowed($_) } keys %{$IkiWiki::hooks{htmlize}}) {
-			push @page_types, [$key, $IkiWiki::hooks{htmlize}{$key}{longname} || $key]
-				unless $IkiWiki::hooks{htmlize}{$key}{nocreate};
+			push @page_types, [$key, $IkiWiki::hooks{htmlize}{$key}{longname} || $key];
 		}
 	}
 	@page_types=sort @page_types;
@@ -557,12 +556,11 @@ sub editcomment ($$) {
 		}
 		
 		$postcomment=1;
-		my $ok=IkiWiki::check_content(
-			content => scalar $form->field('editcontent'),
-			subject => scalar $form->field('subject'),
+		my $ok=IkiWiki::check_content(content => $form->field('editcontent'),
+			subject => $form->field('subject'),
 			$config{comments_allowauthor} ? (
-				author => scalar $form->field('author'),
-				url => scalar $form->field('url'),
+				author => $form->field('author'),
+				url => $form->field('url'),
 			) : (),
 			page => $location,
 			cgi => $cgi,
@@ -602,7 +600,7 @@ sub editcomment ($$) {
 				length $form->field('subject')) {
 				$message = sprintf(
 					gettext("Added a comment: %s"),
-					scalar $form->field('subject'));
+					$form->field('subject'));
 			}
 
 			IkiWiki::rcs_add($file);
