@@ -5,7 +5,6 @@ use warnings;
 use strict;
 use IkiWiki;
 use Encode;
-use File::Path qw{remove_tree};
 use URI::Escape q{uri_escape_utf8};
 use open qw{:utf8 :std};
 
@@ -154,57 +153,23 @@ sub genwrapper {
 	}
 }
 
-my @git_dir_stack;
-my $prefix;
+my $git_dir=undef;
+my $prefix=undef;
 
 sub in_git_dir ($$) {
-	unshift @git_dir_stack, shift;
+	$git_dir=shift;
 	my @ret=shift->();
-	shift @git_dir_stack;
+	$git_dir=undef;
 	$prefix=undef;
 	return @ret;
 }
 
-# Loosely based on git-new-workdir from git contrib.
-sub create_temp_working_dir ($$) {
-	my $rootdir = shift;
-	my $branch = shift;
-	my $working = "$rootdir/.git/ikiwiki-temp-working";
-	remove_tree($working);
-
-	foreach my $dir ("", ".git") {
-		if (!mkdir("$working/$dir")) {
-			error("Unable to create $working/$dir: $!");
-		}
-	}
-
-	# Hooks are deliberately not included: we will commit to the temporary
-	# branch that is used in the temporary working tree, and we don't want
-	# to run the post-commit hook there.
-	#
-	# logs/refs is not included because we don't use the reflog.
-	# remotes, rr-cache, svn are similarly excluded.
-	foreach my $link ("config", "refs", "objects", "info", "packed-refs") {
-		if (!symlink("../../$link", "$working/.git/$link")) {
-			error("Unable to create symlink $working/.git/$link: $!");
-		}
-	}
-
-	open (my $out, '>', "$working/.git/HEAD") or
-		error("failed to write $working.git/HEAD: $!");
-	print $out "ref: refs/heads/$branch\n" or
-		error("failed to write $working.git/HEAD: $!");
-	close $out or
-		error("failed to write $working.git/HEAD: $!");
-	return $working;
-}
-
-sub safe_git {
+sub safe_git (&@) {
 	# Start a child process safely without resorting to /bin/sh.
 	# Returns command output (in list content) or success state
 	# (in scalar context), or runs the specified data handler.
 
-	my %params = @_;
+	my ($error_handler, $data_handler, @cmdline) = @_;
 
 	my $pid = open my $OUT, "-|";
 
@@ -213,20 +178,15 @@ sub safe_git {
 	if (!$pid) {
 		# In child.
 		# Git commands want to be in wc.
-		if (! @git_dir_stack) {
+		if (! defined $git_dir) {
 			chdir $config{srcdir}
 			    or error("cannot chdir to $config{srcdir}: $!");
 		}
 		else {
-			chdir $git_dir_stack[0]
-			    or error("cannot chdir to $git_dir_stack[0]: $!");
+			chdir $git_dir
+			    or error("cannot chdir to $git_dir: $!");
 		}
-
-		if ($params{stdout}) {
-			open(STDOUT, '>&', $params{stdout}) or error("Cannot reopen stdout: $!");
-		}
-
-		exec @{$params{cmdline}} or error("Cannot exec '@{$params{cmdline}}': $!");
+		exec @cmdline or error("Cannot exec '@cmdline': $!");
 	}
 	# In parent.
 
@@ -241,24 +201,24 @@ sub safe_git {
 
 		chomp;
 
-		if (! defined $params{data_handler}) {
+		if (! defined $data_handler) {
 			push @lines, $_;
 		}
 		else {
-			last unless $params{data_handler}->($_);
+			last unless $data_handler->($_);
 		}
 	}
 
 	close $OUT;
 
-	$params{error_handler}->("'@{$params{cmdline}}' failed: $!") if $? && $params{error_handler};
+	$error_handler->("'@cmdline' failed: $!") if $? && $error_handler;
 
 	return wantarray ? @lines : ($? == 0);
 }
 # Convenient wrappers.
-sub run_or_die ($@) { safe_git(error_handler => \&error, cmdline => \@_) }
-sub run_or_cry ($@) { safe_git(error_handler => sub { warn @_ }, cmdline => \@_) }
-sub run_or_non ($@) { safe_git(cmdline => \@_) }
+sub run_or_die ($@) { safe_git(\&error, undef, @_) }
+sub run_or_cry ($@) { safe_git(sub { warn @_ }, undef, @_) }
+sub run_or_non ($@) { safe_git(undef, undef, @_) }
 
 sub ensure_committer {
 	if (! length $ENV{GIT_AUTHOR_NAME} || ! length $ENV{GIT_COMMITTER_NAME}) {
@@ -465,16 +425,6 @@ sub parse_diff_tree ($) {
 	}
 	shift @{ $dt_ref } if $dt_ref->[0] =~ /^$/;
 
-	$ci{details} = [parse_changed_files($dt_ref)];
-
-	return \%ci;
-}
-
-sub parse_changed_files {
-	my $dt_ref = shift;
-
-	my @files;
-
 	# Modified files.
 	while (my $line = shift @{ $dt_ref }) {
 		if ($line =~ m{^
@@ -492,7 +442,7 @@ sub parse_changed_files {
 			my $status = shift(@tmp);
 
 			if (length $file) {
-				push @files, {
+				push @{ $ci{'details'} }, {
 					'file'      => decode_git_file($file),
 					'sha1_from' => $sha1_from[0],
 					'sha1_to'   => $sha1_to,
@@ -506,7 +456,7 @@ sub parse_changed_files {
 		last;
 	}
 
-	return @files;
+	return \%ci;
 }
 
 sub git_commit_info ($;$) {
@@ -519,7 +469,7 @@ sub git_commit_info ($;$) {
 
 	my @raw_lines = run_or_die('git', 'log', @opts,
 		'--pretty=raw', '--raw', '--abbrev=40', '--always', '-c',
-		'-r', $sha1, '--no-renames', '--', '.');
+		'-r', $sha1, '--', '.');
 
 	my @ci;
 	while (my $parsed = parse_diff_tree(\@raw_lines)) {
@@ -617,10 +567,7 @@ sub rcs_commit (@) {
 	# Check to see if the page has been changed by someone else since
 	# rcs_prepedit was called.
 	my $cur    = git_sha1_file($params{file});
-	my $prev;
-	if (defined $params{token}) {
-		($prev) = $params{token} =~ /^($sha1_pattern)$/; # untaint
-	}
+	my ($prev) = $params{token} =~ /^($sha1_pattern)$/; # untaint
 
 	if (defined $cur && defined $prev && $cur ne $prev) {
 		my $conflict = merge_past($prev, $params{file}, $dummy_commit_msg);
@@ -650,24 +597,18 @@ sub rcs_commit_helper (@) {
 		elsif (defined $params{session}->remote_addr()) {
 			$u=$params{session}->remote_addr();
 		}
-		if (length $u) {
+		if (defined $u) {
 			$u=encode_utf8(IkiWiki::cloak($u));
 			$ENV{GIT_AUTHOR_NAME}=$u;
 		}
-		else {
-			$u = 'anonymous';
-		}
 		if (defined $params{session}->param("nickname")) {
 			$u=encode_utf8($params{session}->param("nickname"));
 			$u=~s/\s+/_/g;
 			$u=~s/[^-_0-9[:alnum:]]+//g;
 		}
-		if (length $u) {
+		if (defined $u) {
 			$ENV{GIT_AUTHOR_EMAIL}="$u\@web";
 		}
-		else {
-			$ENV{GIT_AUTHOR_EMAIL}='anonymous@web';
-		}
 	}
 
 	ensure_committer();
@@ -713,7 +654,7 @@ sub rcs_add ($) {
 
 	ensure_committer();
 
-	run_or_cry('git', 'add', '--', $file);
+	run_or_cry('git', 'add', $file);
 }
 
 sub rcs_remove ($) {
@@ -723,7 +664,7 @@ sub rcs_remove ($) {
 
 	ensure_committer();
 
-	run_or_cry('git', 'rm', '-f', '--', $file);
+	run_or_cry('git', 'rm', '-f', $file);
 }
 
 sub rcs_rename ($$) {
@@ -731,7 +672,7 @@ sub rcs_rename ($$) {
 
 	ensure_committer();
 
-	run_or_cry('git', 'mv', '-f', '--', $src, $dest);
+	run_or_cry('git', 'mv', '-f', $src, $dest);
 }
 
 sub rcs_recentchanges ($) {
@@ -829,11 +770,7 @@ sub rcs_diff ($;$) {
 			if (@lines || $line=~/^diff --git/);
 		return 1;
 	};
-	safe_git(
-		error_handler => undef,
-		data_handler => $addlines,
-		cmdline => ["git", "show", $sha1],
-	);
+	safe_git(undef, $addlines, "git", "show", $sha1);
 	if (wantarray) {
 		return @lines;
 	}
@@ -963,11 +900,11 @@ sub git_parse_changes {
 				die $@ if $@;
 				my $fh;
 				($fh, $path)=File::Temp::tempfile(undef, UNLINK => 1);
-				safe_git(
-					error_handler => sub { error("failed writing temp file '$path': ".shift."."); },
-					stdout => $fh,
-					cmdline => ['git', 'show', $detail->{sha1_to}],
-				);
+				my $cmd = "cd $git_dir && ".
+				          "git show $detail->{sha1_to} > '$path'";
+				if (system($cmd) != 0) {
+					error("failed writing temp file '$path'.");
+				}
 			}
 
 			push @rets, {
@@ -1009,14 +946,10 @@ sub rcs_preprevert ($) {
 	my $rev=shift;
 	my ($sha1) = $rev =~ /^($sha1_pattern)$/; # untaint
 
-	my @undo;      # undo stack for cleanup in case of an error
-
-	ensure_committer();
-
 	# Examine changes from root of git repo, not from any subdir,
 	# in order to see all changes.
 	my ($subdir, $rootdir) = git_find_root();
-	return in_git_dir($rootdir, sub {
+	in_git_dir($rootdir, sub {
 		my @commits=git_commit_info($sha1, 1);
 	
 		if (! @commits) {
@@ -1029,62 +962,7 @@ sub rcs_preprevert ($) {
 			error gettext("you are not allowed to revert a merge");
 		}
 
-		# Due to the presence of rename-detection, we cannot actually
-		# see what will happen in a revert without trying it.
-		# But we can guess, which is enough to rule out most changes
-		# that we won't allow reverting.
 		git_parse_changes(1, @commits);
-
-		my $failure;
-		my @ret;
-		eval {
-			my $branch = "ikiwiki_revert_${sha1}"; # supposed to be unique
-
-			push @undo, sub {
-				run_or_cry('git', 'branch', '-D', $branch) if $failure;
-			};
-			if (run_or_non('git', 'rev-parse', '--quiet', '--verify', $branch)) {
-				run_or_non('git', 'branch', '-D', $branch);
-			}
-			run_or_die('git', 'branch', $branch, $config{gitmaster_branch});
-
-			my $working = create_temp_working_dir($rootdir, $branch);
-
-			push @undo, sub {
-				remove_tree($working);
-			};
-
-			in_git_dir($working, sub {
-				run_or_die('git', 'checkout', '--quiet', '--force', $branch);
-				run_or_die('git', 'revert', '--no-commit', $sha1);
-				run_or_die('git', 'commit', '-m', "revert $sha1", '-a');
-			});
-
-			my @raw_lines;
-			@raw_lines = run_or_die('git', 'diff', '--pretty=raw',
-				'--raw', '--abbrev=40', '--always', '--no-renames',
-				"..${branch}");
-
-			my $ci = {
-				details => [parse_changed_files(\@raw_lines)],
-			};
-
-			@ret = git_parse_changes(0, $ci);
-		};
-		$failure = $@;
-
-		# Process undo stack (in reverse order).  By policy cleanup
-		# actions should normally print a warning on failure.
-		while (my $handle = pop @undo) {
-			$handle->();
-		}
-
-		if ($failure) {
-			my $message = sprintf(gettext("Failed to revert commit %s"), $sha1);
-			error("$message\n$failure\n");
-		}
-
-		return @ret;
 	});
 }
 
@@ -1095,11 +973,11 @@ sub rcs_revert ($) {
 
 	ensure_committer();
 
-	if (run_or_non('git', 'cherry-pick', '--no-commit', "ikiwiki_revert_$sha1")) {
+	if (run_or_non('git', 'revert', '--no-commit', $sha1)) {
 		return undef;
 	}
 	else {
-		run_or_non('git', 'branch', '-D', "ikiwiki_revert_$sha1");
+		run_or_die('git', 'reset', '--hard');
 		return sprintf(gettext("Failed to revert commit %s"), $sha1);
 	}
 }
